/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#include <benchmarks/buffer_manager_workload.h>
#include <storage/buffermanager.h>
#include <storage/random_policy.h>
#include <storage/lru_policy.h>
#include <storage/clock_policy.h>
#include <iostream>

template< typename Policy, std::size_t buffer_size = 12>
void runBufferManagerWorkload(aidbms::benchmarks::BufferManagerWorkload& workload) {
    aidbms::storage::BufferManager<Policy, buffer_size> buffer_manager;
    for(const auto& request : workload) {
        if(request.is_pin()) {
            buffer_manager.pin(request.page_id());
        } else {
            buffer_manager.unpin(request.page_id(), false);
        }
    }
    std::cout << buffer_manager << std::endl << std::endl;
}


int main()
{
    // create a test workload from the specified file
    aidbms::benchmarks::BufferManagerWorkload workload("workloads/posgres_buffer_manager_trace.txt");

    // various BufferManager instances with different eviction policies.
    runBufferManagerWorkload<aidbms::storage::RandomPolicy>(workload);
    runBufferManagerWorkload<aidbms::storage::LRUPolicy>(workload);
    runBufferManagerWorkload<aidbms::storage::ClockPolicy>(workload);

    return 0;
}
