/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include <storage/storage.h>
#include <benchmarks/workload.h>

namespace aidbms::benchmarks {
    /**
     * @brief The PageRequest class represents a workload, that is a sequence of pin and unpin operations.
     */
    class PageRequest {
    public:
        enum Operation {
            Pin, Unpin
        };

        PageRequest(Operation operation, aidbms::storage::PageID page_id) : _operation(operation), _page_id(page_id) {}
        inline bool is_pin() const { return _operation == Pin; }
        inline aidbms::storage::PageID page_id() const { return _page_id; }

    private:
        Operation _operation;
        aidbms::storage::PageID _page_id;
    };

    class BufferManagerWorkload : public Workload<PageRequest>
    {
    public:
        /**
         * @brief BufferManagerWorkload reads and parses the file at the specified path, then creates a test workload for the
         * buffer manager.
         * @param workload_file_name path to the file
         */
        BufferManagerWorkload(const char *workload_file_name);
    };
}
