/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include "buffermanager.h"

namespace aidbms::storage {
    /**
     * @brief The RandomPolicy class
     * Example implementation for a eviction policy.
     * When all frames are occupied, a random page
     * is picked among the unpinned ones.
     * Returns nullptr when no pages are unpinned!
     */
    class RandomPolicy
    {
    public:
        RandomPolicy() = default;
        ~RandomPolicy() = default;

        Frame* evict(std::unordered_map<PageID, Frame*> &locations);
        void onPin(PageID, TimeStamp);
        void onUnpin(PageID, TimeStamp);
    };
}
