/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#include <benchmarks/buffer_manager_workload.h>
#include <string>
#include <fstream>
#include <regex>
#include <iostream>
#include <cstdlib>

using namespace aidbms::benchmarks;

BufferManagerWorkload::BufferManagerWorkload(const char *workload_file_name)
{
    std::regex request_regex("([A-Za-z]+) \\(reln = ([0-9]+).*");

    match_function on_match = [this](std::smatch &match) {
        PageRequest::Operation operation = match.str(1) == "Pin" ? PageRequest::Pin : PageRequest::Unpin;
        aidbms::storage::PageID page = std::atol(match.str(2).c_str());
        this->add(PageRequest(operation, page));
    };

    this->parse(workload_file_name, request_regex, on_match);
}
