/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#include <storage/random_policy.h>

using namespace aidbms::storage;

Frame* RandomPolicy::evict(std::unordered_map<PageID, Frame *> &locations)
{
    // Get a list of all unpinned frames.
    std::vector<Frame*> unpinned_frames;
    for (auto& location : locations) {
        if (location.second->isUnpinned()) {
            unpinned_frames.push_back(location.second);
        }
    }

    // When there is no unpinned frame, we have a problem.
    if (unpinned_frames.size() == 0) {
        return nullptr;
    }

    // Otherwise we randomly pick a frame.
    return unpinned_frames[std::rand() % unpinned_frames.size()];
}

void RandomPolicy::onPin(PageID, TimeStamp)
{
    // nothing to do in this strategy!
}

void RandomPolicy::onUnpin(PageID, TimeStamp)
{
    // nothing to do in this strategy!
}
